export default DataProvider;
/** @import MediaComposition from '../model/MediaComposition.js' */
/**
 * Represents a data provider for constructing URLs and handling requests.
 * @class
 */
declare class DataProvider {
    /**
     * Creates an instance of DataProvider.
     *
     * @param {string} [hostName='il.srgssr.ch'] The base host name for constructing URLs
     */
    constructor(hostName?: string);
    /**
     * Sets the integration layer host name.
     *
     * @param {string} hostName The host name to set
     */
    setIlHost(hostName: string): void;
    baseUrl: string;
    /**
     * Handles requests by constructing URLs and fetching data.
     *
     * This provides unified error handling, regardless of the urlHandler used.
     *
     * @param {Function} urlHandler A function that constructs the URL
     *
     * @returns {Promise<MediaComposition>} A promise with the fetched data
     */
    handleRequest(urlHandler: Function): Promise<MediaComposition>;
    /**
     * Gets the media composition URL by URN.
     *
     * @param {string} urn The URN for the media composition
     *
     * @returns {string} The constructed URL
     */
    mediaCompositionUrlHandler(urn: string): string;
}
//# sourceMappingURL=DataProvider.d.ts.map