export default AkamaiTokenService;
/**
 * @class AkamaiTokenService
 */
declare class AkamaiTokenService {
    /**
     * Get the acl path.
     *
     * @param {URL} streamUrl
     *
     * @returns {String}
     */
    static aclPath(streamUrl: URL): string;
    /**
     * AKAMAI
     *
     * @type {String}
     */
    static get AKAMAI(): string;
    /**
     * Check if the resources are protected by an Akamai token.
     * Keep in mind, as we are using the 'some' function,
     * if the resources have at least one resource
     * protected by a token it returns true!
     *
     * @param {Array.<import('../dataProvider/model/typedef').MainResource>} resources
     *
     * @returns {Boolean}
     */
    static hasToken(resources: Array<import('../dataProvider/model/typedef').MainResource>): boolean;
    /**
     * Check if the token type is AKAMAI.
     *
     * @param {String} tokentype
     *
     * @returns {Boolean}
     */
    static isAkamai(tokentype: string): boolean;
    /**
     * Check if the token type is NONE.
     *
     * @param {String} tokentype
     *
     * @returns {Boolean}
     */
    static isNone(tokentype: string): boolean;
    /**
     * NONE
     *
     * @type {String}
     */
    static get NONE(): string;
    /**
     * Generate the stream URL with the akamai token.
     *
     * @param {import('../dataProvider/model/typedef').MainResource} source
     * @param {String} tokenServerUrl
     *
     * @returns {Promise.<import('../dataProvider/model/typedef').MainResource>}
     */
    static tokenize(source: import('../dataProvider/model/typedef').MainResource, tokenServerUrl: string): Promise<import('../dataProvider/model/typedef').MainResource>;
    /**
     * Generate a token for each source
     *
     * @template {import('../dataProvider/model/typedef').MainResource} T
     * @param {Array.<T>} sources
     * @param {String} tokenServerUrl
     *
     * @returns {Promise.<Array.<T>>}
     */
    static tokenizeSources<T extends import("../dataProvider/model/typedef").MainResource>(sources: T[], tokenServerUrl?: string): Promise<T[]>;
}
//# sourceMappingURL=AkamaiTokenService.d.ts.map